# Test a wide variety of input values to the commonly used mpz operations.
# This test should be run whenever optimizations are made to the handling of
# arguments.

import sys
import gmpy2

if sys.version.startswith('3'):
    intTypes = (int,)
else:
    intTypes = (int, long)


valueList = [0, 1, 2, 3, 4, 5]

for power in (14, 15, 16, 29, 30, 31, 32, 45, 48, 60, 64, 75, 90, 96, 105, 120, 128):
    for i in (-2, -1, 0, 1, 2):
        valueList.append(2**power + i)

valueList.append('123456789012345678901234567890')
valueList.append('10000000000000000000000000000000000000000000000000000000000000000')

testValues = []
mpzValues = []
for i in valueList:
    for t in intTypes:
        testValues.append(t(i))
        testValues.append(-t(i))
    mpzValues.append(gmpy2.mpz(i))
    mpzValues.append(-gmpy2.mpz(i))

testValues.extend(mpzValues)

def test():
    for i in testValues:
        for z in mpzValues:
            # Test all permutations of addition
            assert int(i) + int(z) == i + z, (repr(i), repr(z))
            assert int(z) + int(i) == z + i, (repr(i), repr(z))

            # Test all permutations of subtraction
            assert int(i) - int(z) == i - z, (repr(i), repr(z))
            assert int(z) - int(i) == z - i, (repr(i), repr(z))

            # Test all permutations of multiplication
            assert int(i) * int(z) == i * z, (repr(i), repr(z))
            assert int(z) * int(i) == z * i, (repr(i), repr(z))

            # Test all permutations of floor division
            if z != 0:
                assert int(i) // int(z) == i // z, (repr(i), repr(z))
                assert int(i) % int(z) == i % z, (repr(i), repr(z))
                assert divmod(int(i), int(z)) == divmod(i, z), (repr(i), repr(z))

            if i!=0:
                assert int(z) // int(i) == z // i, (repr(i), repr(z))
                assert int(z) % int(i) == z % i, (repr(i), repr(z))
                assert divmod(int(z), int(i)) == divmod(z,i), (repr(i), repr(z))
    return True
            
if __name__ == "__main__":
    print("Testing combinations of mpz and integer operations.")
    print("This test may take a few minutes.")
    test()
    print("Test successful.")



cdef extern from "gmp.h":
    # gmp integers
    ctypedef long mp_limb_t

    ctypedef struct __mpz_struct:
        int _mp_alloc
        int _mp_size
        mp_limb_t* _mp_d

    ctypedef __mpz_struct mpz_t[1]
    ctypedef __mpz_struct *mpz_ptr
    ctypedef const __mpz_struct *mpz_srcptr

    # gmp rationals
    ctypedef struct __mpq_struct:
        __mpz_struct _mp_num
        __mpz_struct _mp_den

    ctypedef __mpq_struct mpq_t[1]
    ctypedef __mpq_struct *mpq_ptr
    ctypedef const __mpq_struct *mpq_srcptr

    void mpz_set(mpz_t rop, mpz_t op)
    void mpq_set(mpq_ptr rop, mpq_srcptr op)
    void mpq_set_num(mpq_t rational, mpz_t numerator)
    void mpq_set_den(mpq_t rational, mpz_t denominator)


cdef extern from "mpfr.h":
    # mpfr reals
    ctypedef int mpfr_sign_t
    ctypedef long mpfr_prec_t
    ctypedef long mpfr_exp_t

    ctypedef struct __mpfr_struct:
        mpfr_prec_t _mpfr_prec
        mpfr_sign_t _mpfr_sign
        mpfr_exp_t _mpfr_exp
        mp_limb_t* _mpfr_d

    ctypedef __mpfr_struct mpfr_t[1]
    ctypedef __mpfr_struct *mpfr_ptr
    ctypedef const __mpfr_struct *mpfr_srcptr

    ctypedef enum mpfr_rnd_t:
        MPFR_RNDN
        MPFR_RNDZ
        MPFR_RNDU
        MPFR_RNDD
        MPFR_RNDA
        MPFR_RNDF
        MPFR_RNDNA

    mpfr_prec_t mpfr_get_prec(mpfr_t x)
    int mpfr_set(mpfr_t rop, mpfr_t op, mpfr_rnd_t rnd)


cdef extern from "mpc.h":
    # mpc complexes
    ctypedef struct __mpc_struct:
        mpfr_t re
        mpfr_t im

    ctypedef __mpc_struct mpc_t[1];
    ctypedef __mpc_struct *mpc_ptr;
    ctypedef const __mpc_struct *mpc_srcptr;
    ctypedef enum mpc_rnd_t:
        MPC_RNDNN
        MPC_RNDNZ
        MPC_RNDNU
        MPC_RNDND
        MPC_RNDZN
        MPC_RNDZZ
        MPC_RNDZU
        MPC_RNDZD
        MPC_RNDUN
        MPC_RNDUZ
        MPC_RNDUU
        MPC_RNDUD
        MPC_RNDDN
        MPC_RNDDZ
        MPC_RNDDU
        MPC_RNDDD

    mpfr_prec_t mpc_get_prec(mpc_srcptr x)
    void mpc_get_prec2(mpfr_prec_t *pr, mpfr_prec_t *pi, mpc_srcptr x)
    int mpc_set(mpc_ptr rop, mpc_srcptr op, mpc_rnd_t rnd)
    int mpc_set_fr_fr(mpc_ptr rop, mpfr_srcptr rp, mpfr_srcptr ip, mpc_rnd_t rnd)


cdef extern from "gmpy2.h":
    # Initialize the C-API
    # This must be called before any other functions, but not to access
    # the types.
    cdef int import_gmpy2() except -1

    # Object types
    ctypedef class gmpy2.mpz [object MPZ_Object]:
        cdef mpz_t z
    ctypedef class gmpy2.mpq [object MPQ_Object]:
        cdef mpq_t q
    ctypedef class gmpy2.mpfr [object MPFR_Object]:
        cdef mpfr_t f
        cdef int rc
    ctypedef class gmpy2.mpc [object MPC_Object]:
        cdef mpc_t c
        cdef int rc

    # Object creation
    cdef mpz GMPy_MPZ_New(void *)
    cdef mpq GMPy_MPQ_New(void *)
    cdef mpfr GMPy_MPFR_New(mpfr_prec_t prec, void *)
    cdef mpc GMPy_MPC_New(mpfr_prec_t rprec, mpfr_prec_t iprec, void *)

    # C field access
    cdef mpz_t MPZ(mpz)
    cdef mpq_t MPQ(mpq)
    cdef mpfr_t MPFR(mpfr)
    cdef mpc_t MPC(mpc)

    # Type check
    cdef bint MPZ_Check(object)
    cdef bint MPQ_Check(object)
    cdef bint MPFR_Check(object)
    cdef bint MPC_Check(object)


# Build a gmpy2 mpz from a gmp mpz
cdef inline mpz GMPy_MPZ_From_mpz(mpz_srcptr z):
    cdef mpz res = GMPy_MPZ_New(NULL)
    mpz_set(res.z, z)
    return res

# Build a gmpy2 mpq from a gmp mpq
cdef inline mpq GMPy_MPQ_From_mpq(mpq_srcptr q):
    cdef mpq res = GMPy_MPQ_New(NULL)
    mpq_set(res.q, q)
    return res

# Build a gmpy2 mpq from gmp mpz numerator and denominator
cdef inline mpq GMPy_MPQ_From_mpz(mpz_srcptr num, mpz_srcptr den):
    cdef mpq res = GMPy_MPQ_New(NULL)
    mpq_set_num(res.q, num)
    mpq_set_den(res.q, den)
    return res

# Build a gmpy2 mpfr from a mpfr
cdef inline mpfr GMPy_MPFR_From_mpfr(mpfr_srcptr x):
    cdef mpfr res = GMPy_MPFR_New(mpfr_get_prec(x), NULL)
    mpfr_set(res.f, x, MPFR_RNDN)
    return res

# Build a gmpy2 mpc from a mpc
cdef inline mpc GMPy_MPC_From_mpc(mpc_srcptr c):
    cdef mpfr_prec_t pr
    cdef mpfr_prec_t pi
    mpc_get_prec2(&pr, &pi, c)
    cdef mpc res = GMPy_MPC_New(pr, pi, NULL)
    mpc_set(res.c, c, MPC_RNDNN)
    return res

# Build a gmpy2 mpc from a real part mpfr and an imaginary part mpfr
cdef inline mpc GMPy_MPC_From_mpfr(mpfr_srcptr re, mpfr_srcptr im):
    cdef mpc res = GMPy_MPC_New(mpfr_get_prec(re), mpfr_get_prec(im), NULL)
    # We intentionally use MPFR funtions instead of MPC functions here
    # in order not to add an unneeded dependency on MPC. It's probably
    # faster too this way.
    mpfr_set(res.c.re, re, MPFR_RNDN)
    mpfr_set(res.c.im, im, MPFR_RNDN)
    return res



from __future__ import print_function, division

import sys
import os
import glob
import doctest
from doctest import DocTestParser, Example, SKIP
import gmpy2

# *****************************************************************************
# Test strategy
# -------------
# Tests are divided into two different categories:
#
#   1) The 'txt' files contain doctest style tests. These tests should cover
#      basic functionality for all functions/types.
#   2) The 'py' files contain Python code that perform extensive tests, but
#      may not test every function.
#
# If run by a debug build of Python, the test suite can be repeated multiple
# times to search for memory leaks.
#
# NOTE: IF THE LAST TEST IN A BLOCK OF TESTS GENERATES AN EXCEPTION, THE
#       REFERENCE COUNTING IN A DEBUG BUILD GETS CONFUSED. ALWAYS ENSURE THAT
#       AT LEAST ONE VALID TEST IS PERFORMED AFTER AN EXCEPTION IS RAISED!
#
# *****************************************************************************

# Check if this is a debug build of Python.
try:
    sys.gettotalrefcount()
    debug = True
except AttributeError:
    debug = False

# Change repeat to the number of times to repeat each test. Combined with a
# debug build, this can help identify memory leaks.
if debug:
    try:
        repeat = abs(int(sys.argv[1]))
    except:
        repeat = 1
else:
    repeat = 1

# If mpc version < 1.1.0 gmpy2.root_of_unity is defined and gmpy2.cmp_abs
# doesn't manage complex parameters.
# We create a doctest flag to skip a doctest when mpc version is < 1.1.0
SKIP_MPC_LESS_THAN_110 = doctest.register_optionflag("SKIP_MPC_LESS_THAN_110")
mpc_version_110 = 'root_of_unity' in dir(gmpy2) # True if mpc version >= 1.1.0

SKIP_IN_DEBUG_MODE = doctest.register_optionflag("SKIP_IN_DEBUG_MODE")

class Gmpy2DocTestParser(DocTestParser):
    def parse(self, *args, **kwargs):
        examples = DocTestParser.parse(self, *args, **kwargs)
        for example in examples:
            if not isinstance(example, Example):
                continue
            if not mpc_version_110 and SKIP_MPC_LESS_THAN_110 in example.options:
                example.options[SKIP] = True
            if debug and SKIP_IN_DEBUG_MODE in example.options:
                example.options[SKIP] = True

        return examples

parser = Gmpy2DocTestParser()

print()
print("Unit tests for gmpy2 {0} with Python {1}".format(gmpy2.version(), sys.version.split()[0]))
print("  Mutliple-precision library:     {0}".format(gmpy2.mp_version()))
print("  Floating-point library:         {0}".format(gmpy2.mpfr_version()))
print("  Complex library:                {0}".format(gmpy2.mpc_version()))
print("  Caching Values: (Cache size)    {0}".format(gmpy2.get_cache()[0]))
print("  Caching Values: (Size in limbs) {0}".format(gmpy2.get_cache()[1]))
print()

mpz_doctests = ["test_mpz_create.txt", "test_mpz.txt", "test_mpz_io.txt",
                "test_mpz_pack_unpack.txt", "test_misc.txt"]

mpq_doctests = ["test_mpq.txt"]

mpfr_doctests = ["test_mpfr_create.txt", "test_mpfr.txt",
                 "test_mpfr_trig.txt", "test_mpfr_min_max.txt",
                 "test_context.txt", "test_mpfr_subnormalize.txt"]

# Some tests may differ between MPFR3 and MPFR4.
mpfr_major_version = gmpy2.mpfr_version().split()[1].split('.')[0]
mpfr_version_tests = [os.path.basename(i)
                      for i in glob.glob(os.path.join(os.path.dirname(__file__),
                                         "test_mpfr" + mpfr_major_version + "*.txt"))]

mpc_doctests = ["test_mpc_create.txt", "test_mpc.txt", "test_mpc_trig.txt"]

gmpy2_tests = [os.path.basename(i)
               for i in glob.glob(os.path.join(os.path.dirname(__file__),
                                  "test_gmpy2*.txt"))]

# The following tests will only pass on Python 3.2+.
py32_doctests = ["test_py32_hash.txt"]

failed = 0
attempted = 0

all_doctests = gmpy2_tests + mpz_doctests + mpq_doctests

all_doctests += mpfr_doctests + mpfr_version_tests

all_doctests += mpc_doctests

if sys.version_info > (3,1):
    all_doctests += py32_doctests

for test in sorted(all_doctests):
    if test.endswith("py2.txt") and sys.version_info[0] >= 3:
        continue
    if test.endswith("py3.txt") and sys.version_info[0] < 3:
        continue
    for r in range(repeat):
        result = doctest.testfile(test, globs=globals(),
                                  optionflags=doctest.IGNORE_EXCEPTION_DETAIL |
                                              doctest.NORMALIZE_WHITESPACE |
                                              doctest.REPORT_NDIFF,
                                  parser=parser)
        print("Results for:  {0:25}".format(test.split(".")[0]), end="")
        print(" Attempted: {1:4d}   Failed: {0:4d}".format(*result), end="")
        if debug:
            print(" RefCount: {0:6d}".format(sys.gettotalrefcount()))
        else:
            print()
        failed += result[0]
        attempted += result[1]
    if repeat > 1:
        print()


print()
print("                              Summary - Attempted: {0:4d}   Failed: {1:4d}".format(attempted, failed))
print()
print("Running external test programs.")

print("Running {0:30}  ".format("test_pack.py"), end="")
import test_pack
if test_pack.test():
    print("successful")
    attempted += 1
else:
    print("failed")
    failed += 1

print("Running {0:30}  ".format("test_mpz_args.py"), end="")
import test_mpz_args
if test_mpz_args.test():
    print("successful")
    attempted += 1
else:
    print("failed")
    failed += 1

if failed:
    sys.exit(1)
else:
    sys.exit(0)



from __future__ import print_function
import gmpy2

def test(repeat=1):
    """Test gmpy2.pack and gmpy2.unpack."""
    r = gmpy2.random_state(42)
    try:
        for counter in range(repeat):
            for t in (10, 30, 60, 500, 1000, 2000, 10000, 100000):
                v = gmpy2.mpz_rrandomb(r, t)
                for b in range(1, max(1001,t)):
                    temp = gmpy2.unpack(v, b)
                    u = gmpy2.pack(temp, b)
                    if u != v:
                        raise ValueError
        return True
    except ValueError:
        return False


if __name__ == "__main__":
    import sys
    try:
        repeat = abs(int(sys.argv[1]))
    except:
        repeat = 1
    print("Testing pack/unpack for a large number of values.")
    print("This test may take a few minutes.")
    test(repeat)
    print("Test successful.")
